@@ -1,6 +1,8 @@
 // -*- mode: C++; c-file-style: "cc-mode" -*-
 //*************************************************************************
 //
+// Code available from: https://verilator.org
+//
 // Copyright 2003-2021 by Wilson Snyder. This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
@@ -10,30 +12,27 @@
 //*************************************************************************
 ///
 /// \file
-/// \brief Verilator: Include to provide information about symbol inspection
+/// \brief Verilated symbol inspection header
 ///
-///     This file is for inclusion by internal files that need to inspect
-///     specific symbols.
+/// This file is for inclusion by internal files that need to inspect
+/// specific symbols.  Applications typically use the VPI instead.
 ///
-///     User routines wanting to inspect the symbol table should use
-///     verilated_syms.h instead.
-///
-///     These classes are thread safe, and read only.
-///
-/// Code available from: https://verilator.org
+/// User wrapper code wanting to inspect the symbol table should use
+/// verilated_syms.h instead.
 ///
 //*************************************************************************
+// These classes are thread safe, and read only.
 
-#ifndef _VERILATED_SYM_PROPS_H_
-#define _VERILATED_SYM_PROPS_H_ 1  ///< Header Guard
+#ifndef VERILATOR_VERILATED_SYM_PROPS_H_
+#define VERILATOR_VERILATED_SYM_PROPS_H_
 
 #include "verilatedos.h"
 
 #include <vector>
 
 //===========================================================================
-/// Verilator range
-/// Thread safety: Assume is constructed only with model, then any number of readers
+// Verilator range
+// Thread safety: Assume is constructed only with model, then any number of readers
 
 // See also V3Ast::VNumRange
 class VerilatedRange final {
@@ -65,12 +64,12 @@ public:
 };
 
 //===========================================================================
-/// Verilator variable
-/// Thread safety: Assume is constructed only with model, then any number of readers
+// Verilator variable
+// Thread safety: Assume is constructed only with model, then any number of readers
 
 class VerilatedVarProps VL_NOT_FINAL {
     // TYPES
-    enum { MAGIC = 0xddc4f829 };
+    static constexpr vluint32_t MAGIC = 0xddc4f829UL;
     // MEMBERS
     const vluint32_t m_magic;  // Magic number
     const VerilatedVarType m_vltype;  // Data type
@@ -145,7 +144,7 @@ public:
     }
     vluint32_t entSize() const;
     bool isPublicRW() const { return ((m_vlflags & VLVF_PUB_RW) != 0); }
-    /// DPI compatible C standard layout
+    // DPI compatible C standard layout
     bool isDpiCLayout() const { return ((m_vlflags & VLVF_DPI_CLAY) != 0); }
     int udims() const { return m_udims; }
     int dims() const { return m_pdims + m_udims; }
@@ -153,39 +152,43 @@ public:
     const VerilatedRange& unpacked() const { return m_unpacked[0]; }
     // DPI accessors
     int left(int dim) const {
-        return dim == 0 ? m_packed.left()
-                        : VL_LIKELY(dim >= 1 && dim <= udims()) ? m_unpacked[dim - 1].left() : 0;
+        return dim == 0                                ? m_packed.left()
+               : VL_LIKELY(dim >= 1 && dim <= udims()) ? m_unpacked[dim - 1].left()
+                                                       : 0;
     }
     int right(int dim) const {
-        return dim == 0 ? m_packed.right()
-                        : VL_LIKELY(dim >= 1 && dim <= udims()) ? m_unpacked[dim - 1].right() : 0;
+        return dim == 0                                ? m_packed.right()
+               : VL_LIKELY(dim >= 1 && dim <= udims()) ? m_unpacked[dim - 1].right()
+                                                       : 0;
     }
     int low(int dim) const {
-        return dim == 0 ? m_packed.low()
-                        : VL_LIKELY(dim >= 1 && dim <= udims()) ? m_unpacked[dim - 1].low() : 0;
+        return dim == 0                                ? m_packed.low()
+               : VL_LIKELY(dim >= 1 && dim <= udims()) ? m_unpacked[dim - 1].low()
+                                                       : 0;
     }
     int high(int dim) const {
-        return dim == 0 ? m_packed.high()
-                        : VL_LIKELY(dim >= 1 && dim <= udims()) ? m_unpacked[dim - 1].high() : 0;
+        return dim == 0                                ? m_packed.high()
+               : VL_LIKELY(dim >= 1 && dim <= udims()) ? m_unpacked[dim - 1].high()
+                                                       : 0;
     }
     int increment(int dim) const {
-        return dim == 0
-                   ? m_packed.increment()
-                   : VL_LIKELY(dim >= 1 && dim <= udims()) ? m_unpacked[dim - 1].increment() : 0;
+        return dim == 0                                ? m_packed.increment()
+               : VL_LIKELY(dim >= 1 && dim <= udims()) ? m_unpacked[dim - 1].increment()
+                                                       : 0;
     }
     int elements(int dim) const {
-        return dim == 0
-                   ? m_packed.elements()
-                   : VL_LIKELY(dim >= 1 && dim <= udims()) ? m_unpacked[dim - 1].elements() : 0;
+        return dim == 0                                ? m_packed.elements()
+               : VL_LIKELY(dim >= 1 && dim <= udims()) ? m_unpacked[dim - 1].elements()
+                                                       : 0;
     }
-    /// Total size in bytes (note DPI limited to 4GB)
+    // Total size in bytes (note DPI limited to 4GB)
     size_t totalSize() const;
-    /// Adjust a data pointer to access a given array element, NuLL if something goes bad
+    // Adjust a data pointer to access a given array element, NuLL if something goes bad
     void* datapAdjustIndex(void* datap, int dim, int indx) const;
 };
 
 //===========================================================================
-/// Verilator DPI open array variable
+// Verilator DPI open array variable
 
 class VerilatedDpiOpenVar final {
     // MEMBERS
@@ -222,8 +225,8 @@ public:
 };
 
 //===========================================================================
-/// Verilator variable
-/// Thread safety: Assume is constructed only with model, then any number of readers
+// Verilator variable
+// Thread safety: Assume is constructed only with model, then any number of readers
 
 class VerilatedVar final : public VerilatedVarProps {
     // MEMBERS
